/* Functions which are called exclusively by the eMusic Toolbar itself */

function eMusic_SetupDailyDownload()
{
	var ddFrame = document.getElementById("eMusic-DailyDownload-Frame");
 	var ddDoc = ddFrame.contentDocument;

	if( ddFrame.currentURI.asciiSpec != eMusic_DailyDownloadPopupURL )
	{
		ddFrame.loadURI(eMusic_DailyDownloadPopupURL);
	}
	else
	{
		var list = ddDoc.getElementsByTagName("A");

		for( var i = 0; i < list.length; i++ )
		{
			list[i].addEventListener("click", eMusic_DailyDownloadClick, true);
		}		
	}	
}

function eMusic_DailyDownloadClick(event)
{
	var target = event.target;
	var relativeURL;
	var URL;

	if(target.tagName == "A")
	{
		relativeURL = target.getAttribute("href");
		URL = eMusic_BaseURL + relativeURL;
		eMusic_LoadURL(URL);
	}
	else if(target.tagName == "IMG")
	{
		// I don't understand why the parentNode attribute holds the URL...
		// but it works, so I'm not going to complain right now
		eMusic_LoadURL(event.target.parentNode);
	}
	event.stopPropagation();
	event.preventDefault();
}

function eMusic_ShowOptionsDialog()
{
	window.openDialog(eMusic_OptionsDialog,"eMusicOptions","chrome,centerscreen");
}

function eMusic_Initialize()
{
	var searchBox = document.getElementById("eMusic-SearchTerms");
	var tb = document.getElementById("eMusic-Toolbar");
	var ddFrame = document.getElementById("eMusic-DailyDownload-Frame");

	if(!eMusic_Initialized)
	{
		window.removeEventListener("focus", eMusic_Initialize, true);
	
		for(var i = 0; i < eMusic_HideableItems.length; i++)
		{
			eMusic_SetHidden(eMusic_HideableItems[i]);
		}

		tb.setAttribute("labels", eval(eMusic_TB_ShowText));

		ddFrame.addEventListener("load", eMusic_SetupDailyDownload, true);

		eMusic_PrefObserver.register();

		eMusic_Initialized = 1;
	}
}

function eMusic_TrimString(string)
{
	// Return empty if nothing was passed in
	if (!string) return "";
	
	// Efficiently replace any leading or trailing whitespace
	var value = string.replace(/^\s+/, '');
	value = string.replace(/\s+$/, '');
	
	// Replace any multiple whitespace characters with a single space
	value = value.replace(/\s+/g, ' ');
	
	// Return the altered string
	return value;
}

function eMusic_ConvertTermsToURI(terms)
{
	// Split up the search terms based on the space character
	var termArray = new Array();
	termArray = terms.split(" ");
	var result = "";

	// Loop through the search terms, building up the result string
	for(var i = 0; i < termArray.length; i++)
	{
		if(i > 0) 
		{ 
			result += "+"; 
		}
		
		result += encodeURIComponent(termArray[i]);
	}
	
	// Return the result
	return result;
}

function eMusic_Populate()
{
	// Get the menu element that we will be working with
	var menu = document.getElementById("eMusic-SearchTermsMenu");

	// Clean up whatever is currently in the menu
	for(var i = menu.childNodes.length - 1; i>=0; i--) 
	{
		menu.removeChild(menu.childNodes.item(i));
	}

	// Load the search terms into our menu
	for(var i=0; i < eMusic_MaxSavedSearches; i++)
	{
		// For each search term, create a menu item element
		var tempItem = null;
		tempItem = document.createElement("menuitem");

		var itemLabel = eMusic_Branch.getCharPref("savedsearch"+i);

		// Set the menuitem element's various attributes:
		// The label will be the search term itself
		if (itemLabel)
		{
			tempItem.setAttribute("label", itemLabel);
			tempItem.setAttribute("oncommand", "eMusic_SearchListCommand(event);");

			// Add the item to our menu
			menu.appendChild(tempItem);
		}
	}
}

function eMusic_SearchListCommand(event)
{
	var item = event.target;

	if(eMusic_TB_AutoSearch)
		eMusic_Search('default');
	else
		eMusic_UpdateSavedSearches(item.label);
}

function eMusic_UpdateSavedSearches(newQuery)
{
	if(!eMusic_TB_SearchHistory) return;

	var firstMatch = eMusic_MaxSavedSearches;
	var LastSavedSearch = eMusic_MaxSavedSearches;

	for(var i = 0; i < eMusic_MaxSavedSearches; i++)
	{
		var savedItem = eMusic_Branch.getCharPref("savedsearch"+i);

		if( !savedItem)
		{
			LastSavedSearch = i-1;
			break;
		}	
				
		if(newQuery == savedItem)
		{
			firstMatch = i;
			break;
		}			
	}

	for(var i = Math.min(firstMatch, (LastSavedSearch+1)); i > 0; i--)
	{
		eMusic_Branch.setCharPref("savedsearch"+i, eMusic_Branch.getCharPref("savedsearch"+(i-1)));
	}

	eMusic_Branch.setCharPref("savedsearch0", newQuery);
}

function eMusic_ChangeSearchTarget(newTarget)
{
	eMusic_Branch.setCharPref("options.searchemusic", newTarget);
	if(eMusic_TB_ShowText)
	{
		// Get a handle to our default search menu
		var searchMenu = document.getElementById("eMusic-SearchMenu");
		
		var localeStrings = document.getElementById("eMusic-Bundle");
		var labelName = "eMusic-Search-" + newTarget + "-Label";
		searchMenu.setAttribute("label", localeStrings.getString(labelName));
	}
}

function eMusic_Search()
{
	// Create a few variables
	var URL = "";
	var isEmpty = false;
	var type = eMusic_Branch.getCharPref("options.searchemusic");
	var mode;
	
	if(!type)
		type = "eMusic";

	// Get a handle to our search terms box
	var searchTermsBox = document.getElementById("eMusic-SearchTerms");

	// Get a handle to our default search menu
	var searchMenu = document.getElementById("eMusic-SearchMenu");

	if( !searchMenu ) return;
    
	// Get the value in the search terms box, trimming whitespace as necessary
	var searchTerms = eMusic_TrimString(searchTermsBox.value);
	searchTermsBox.value = "";

	// If there are no search terms, than we set isEmpty to true
	// Otherwise, we convert the search terms to a safe URI version
	if(searchTerms.length == 0)
	{
		isEmpty = true;
	}
	else
	{
		eMusic_UpdateSavedSearches(searchTerms);
		searchTerms = eMusic_ConvertTermsToURI(searchTerms);
	}
	
	if(isEmpty)
		URL = "http://www.emusic.com/search.html";
	else if(type == "Forums")
	{
		URL = "http://www.emusic.com/messageboard/search.html?forum=0&scope=0&query=" + searchTerms;
	}
	else
	{
		// Do something different for differing search types
		switch(type)
		{
		case "Artist":
			mode = "a";
			break;

		case "Album":
			mode = "b";
			break;

		case "Track":
			mode = "s";
			break;

		case "Label":
			mode = "l";
			break;

		case "Classical":
			mode = "cx";
			break;

		case "Composer":
			mode = "c";
			break;

		case "CAlbum":
			mode = "ca";
			break;

		case "Work":
			mode = "cw";
			break;

		case "Performer":
			mode = "cp";
			break;
		
		case "Conductor":
			mode = "cc";
			break;		

		case "Ensemble":
			mode = "ce";
			break;
		
		// Build up the URL for an eMusic search
		case "eMusic":
		default:
			mode = "x"
		}
		
		URL = "http://www.emusic.com/search.html?mode=" + mode + "&QT=" + searchTerms;	
	}

	// Load the URL in the browser window using the eMusic_LoadURL function
	if(eMusic_TB_NewWindow)
		eMusic_LoadURLNew(URL);
	else
		eMusic_LoadURL(URL);
}